package exosite;  
import java.io.BufferedReader; 
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import com.ewon.ewonitf.EWException;
import com.ewon.ewonitf.ScheduledActionManager;
import com.ewon.ewonitf.SysControlBlock;
import com.hms_networks.americas.sc.datapoint.DataPoint;
import com.hms_networks.americas.sc.datapoint.DataType;

public class ExositeService {
	
	private static final String token_file = "/usr/exosite_token2.txt";
	private static final String cfg_file = "/usr/config.txt";
	
	private static final String response_file = "/usr/reponse.txt";
	private static final String data_in_response_file = "/usr/data_in_response_file.txt";
	private static final String config_io_response_file = "/usr/config_io_response.txt";
	private static final String data_out_response_file = "/usr/data_out_response.txt";
	
	private String product_domain_start = "https://";
	private String provision_endpoint = "/provision/activate";
	private String data_in_endpoint = "/onep:v1/stack/alias";
	private String historical_in_endpoint = "/onep:v1/stack/record";
	private String read_configio_endpoint = "/onep:v1/stack/alias?config_io";
	
	private String default_polling_rate = "120";

	
	public void saveToken(String token)
	{
		try 
		{
			BufferedWriter writer = new BufferedWriter(new FileWriter(token_file));
			writer.write(token);
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void clear_config_io_response() throws IOException
	{
		File file = new File(config_io_response_file);
		if (file.exists())
			file._delete();
	}
	
	public String readToken()
	{
		String token = "";
		
		File checkFile = new File(token_file);
		if (!checkFile.exists()) return token;
		
		BufferedReader reader;
		try {
			reader = new BufferedReader(new FileReader(token_file));
			token = reader.readLine();
		} catch (Exception e) {
			e.printStackTrace();
		}
		 
		return token;
	}
	
	public String[] readConfig()
	{
		String[] params = new String[2];
		params[0] = "";
		params[1] = default_polling_rate;
		File checkFile = new File(cfg_file);
		if (!checkFile.exists()) return params;
		
		BufferedReader reader;
		try {
			reader = new BufferedReader(new FileReader(cfg_file));
			reader.readLine();
			String sid_str = reader.readLine();
			params[0] = sid_str.substring(6, sid_str.length());
			String pr_str = reader.readLine();
			params[1] = pr_str.substring(15, pr_str.length());
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return params;
	}
	
	public String provision(String exo_host)
	{
		String token = "";

		String URL = product_domain_start + exo_host + provision_endpoint;
		int res = ResponseCode.NO_ERROR;
		String id = "";
		
		SysControlBlock Sys;
		
		try {
		  Sys = new SysControlBlock(SysControlBlock.INF);
		  id = Sys.getItem("SerNum");
	    } catch (EWException e) {
	      res = ResponseCode.EWON_ERROR;
	    }
		
		System.out.println("Provisioning on" + URL + "with " + id);
		
		try {
			res = ScheduledActionManager.RequestHttpX(URL, "POST", "", "id=" + id, "", response_file);
		
			
		} catch (EWException e ) {
			res = ResponseCode.EWON_ERROR;
		}
	    	    
	    if (res == ResponseCode.NO_ERROR)
		{
		    BufferedReader reader;
		    try {
				reader = new BufferedReader(new FileReader(response_file));
			    token = reader.readLine();
			    saveToken(token);
			} catch (Exception e) {
				e.printStackTrace();
			}
	    }
	    System.out.println("response: " + token);
	    return token;    
	}
	
	private void printFile(String filename)
	{
		File file = new File(filename);
		if (!file.exists()) return;
		BufferedReader reader;
		try {
			reader = new BufferedReader(new FileReader(filename));
			System.out.println(filename + " contents: " + reader.readLine());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public int syncConfigIO(String token, String exo_host)
	{
		String URL = product_domain_start + exo_host + read_configio_endpoint;  
		String auth_header = "X-Exosite-CIK=" + token;
		String accept_header="Accept=application/x-www-form-urlencoded";
		String if_modified_since_header="If-Modified-Since=0";
		String timeout_header ="Request-Timeout=2";
		int res = ResponseCode.NO_ERROR;
		
		try {
			clear_config_io_response();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		String headers = if_modified_since_header + "&" + timeout_header + "&" + auth_header + "&" + accept_header;
		
		try {
			res = ScheduledActionManager.RequestHttpX(URL, "GET", headers, "", "", config_io_response_file);
		} catch (EWException e) {
			res = ResponseCode.EWON_ERROR;
		}
		
		File file = new File(config_io_response_file);
		
		System.out.println("res: "+ res);
		if (res == ResponseCode.NO_ERROR && file.exists())
		{
			BufferedReader reader;
			try {
				reader = new BufferedReader(new FileReader(config_io_response_file));
				String config_io = reader.readLine();
				headers=auth_header;
				System.out.println("Updating config_io with: " + headers);
				res = ScheduledActionManager.RequestHttpX(URL, "POST", headers, config_io, "", config_io_response_file);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		else
		{
			System.out.println("ConfigIO unchanged");
		}
		return 0;
		
	}
	
	public long longp(String token, String exo_host, long polltime, long ifmodifiedsince)
	{
		
		String URL = product_domain_start + exo_host + "onep:v1/stack/alias?data_out";  
		String auth_header = "X-Exosite-CIK=" + token;
		String accept_header="Accept=application/x-www-form-urlencoded";
		String if_modified_since_header="If-Modified-Since="+ifmodifiedsince;
		String timeout_header ="Request-Timeout="+polltime;
		int res = ResponseCode.NO_ERROR;
		
		try {
			clear_config_io_response(); 
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		String headers = if_modified_since_header + "&" + timeout_header + "&" + auth_header + "&" + accept_header;

	    try {
		  res = ScheduledActionManager.RequestHttpX(URL, "GET", headers, "", "", data_out_response_file);
	    } catch (EWException e) {
	      res = ResponseCode.EWON_ERROR;
	    }
	    
	    File file = new File(data_out_response_file);
		
		if (res == ResponseCode.NO_ERROR && file.exists())
		{
			BufferedReader reader;
			try {
				reader = new BufferedReader(new FileReader(data_out_response_file));
				String data_out = reader.readLine();
				headers=auth_header;
			} catch (Exception e) {
				e.printStackTrace();
			}
			try {
				FileWriter fw = new FileWriter(data_out_response_file);
				fw.write("--");
			} catch (Exception e ) {
				System.out.println("An error occured");
			}
		}
		else
		{
			System.out.println("No data_out");
		}
	    System.out.println("response: "+res);
	    
	    return System.currentTimeMillis() / 1000;
	}
	
	public int writeData(String data, String token, String exo_host)
	{
		String URL = product_domain_start + exo_host + data_in_endpoint;
		int res = ResponseCode.NO_ERROR;
		
		String headers = "X-Exosite-CIK=" + token;

	    try {
		  res = ScheduledActionManager.RequestHttpX(URL, "POST", headers, "data_in=" + data,"", data_in_response_file);
	    } catch (EWException e) {
	      res = ResponseCode.EWON_ERROR;
	    }
	    
	    return res;
	}
	
	private boolean is_previous_complete_data_in(String currentTimestamp, String previousTimestamp, String data_in)
	{
		// data_in is 'empty' so it's not complete
		if (data_in == "{") return false;
		
		//If there's a new timestamp, the previous timestamp is completed,
		// Or, the index is the size, so we're on the last DataPoint
		if (currentTimestamp.compareTo(previousTimestamp) == 0) {
			return false;
		} else {
			return true;
		}
		
	}
	
	private boolean is_current_complete_data_in(int index, int size)
	{
		return index == size-1;
	}
	
	private String addDataPointToDataIn(String data_in, DataPoint dp)
	{
		if (data_in == "") 
			data_in = "{";
		//Add current datapoint
		if (data_in != "{") data_in = data_in + ",";
		String value = dp.getValueString();
		DataType d = dp.getType();
		if (d.getRawDataType() == 4) {
			value = '\"' + value + '\"';
		}
		data_in = data_in + "\"" + dp.getTagId() + "\"" + ":" + value;
		
		return data_in;
	}
	
	private String addDataInToPayload(String payload, String data_in, String data_in_timestamp)
	{
		String timestamp_us = Long.toString(Long.parseLong(data_in_timestamp));
		System.out.print("timestamp: " + timestamp_us + " ");
		data_in = data_in + "}"; //Close data_in 'object'
		payload = payload + "&" + timestamp_us + "=" + data_in;
		return payload;
	}
	
	public int writeDataRealtime(String data_in, String token, String exo_host)
	{
		String URL = product_domain_start + exo_host + data_in_endpoint; 
		int res = ResponseCode.NO_ERROR;
		
		String headers = "Accept: */*" + "&" + "X-Exosite-CIK="+ token;
		
		String payload = "data_in=" + data_in;
		System.out.print(payload);
		try {
    			res = ScheduledActionManager.RequestHttpX(URL, "POST", headers, payload,"", data_in_response_file);
    			System.out.print(" response: " + res + " ");
    		} catch (EWException e) {
    			res = ResponseCode.EWON_ERROR;
    		}
		
		return res;
		
	}
	public int writeHistoricalData(ArrayList data, String token, String exo_host)
	{
		String URL = product_domain_start + exo_host + historical_in_endpoint; 
		int res = ResponseCode.NO_ERROR;
		
		String headers = "Accept: */*" + "&" + "X-Exosite-CIK="+token;
		
		String payload = "alias=data_in";
		
		String data_in_timestamp = ""; //Just a default
		String data_in = "";
		
		//<time1>={"001":1234}& 
		for(int i=0; i<data.size(); i++)
		{
			DataPoint dp = (DataPoint) data.get(i);
			if (data_in_timestamp == "") data_in_timestamp = getTimestampFromDP(dp);//set data_in ts
			if (is_previous_complete_data_in(getTimestampFromDP(dp), data_in_timestamp, data_in)) {
				payload = addDataInToPayload(payload, data_in, data_in_timestamp);
				data_in = ""; //Start fresh data_in 'object'
				data_in_timestamp = getTimestampFromDP(dp);
				data_in = addDataPointToDataIn(data_in, dp);
			} else if (is_current_complete_data_in(i, data.size())) {
				data_in = addDataPointToDataIn(data_in, dp);
				payload = addDataInToPayload(payload, data_in, data_in_timestamp);
				data_in = "";
				data_in_timestamp = ""; //reset so as to re-initialize with the next dp timestamp
			} else {
				data_in = addDataPointToDataIn(data_in, dp); 
			}
		}
		//Payload format:
		//alias=<alias 1>&<timestamp 1>=<value 1>&<timestamp 2>=<value 2>&alias=<alias 2>&<timestamp 3>=<value 3>&<timestamp 4>=<value 4>
	    System.out.print("historical payload: " + payload);
		if (data.size() > 0)
	    {
	    	try {
	    		res = ScheduledActionManager.RequestHttpX(URL, "POST", headers, payload,"", data_in_response_file);
	    		System.out.print(" response: " + res + " ");
	    	} catch (EWException e) {
	    		res = ResponseCode.EWON_ERROR;
	    	}
	    }
	    else
	    {
	    	System.out.println("No new data points");  
	    }
	    return res;
	}
	
	private String getTimestampFromDP(DataPoint dp)
	{
		String longTimestamp = dp.getTimeStamp();
		int timestamp_int = Integer.parseInt(longTimestamp);
		return timestamp_int + "";
	}
}

